from __future__ import print_function

# -*- coding: utf-8 -*-

import contextlib
import os
import sys
import time


class Timer(object):
    def __init__(self):
        self.start_time = time.time()
        self.start_clock = self._clock()

    def _clock(self):
        times = os.times()
        # HACK, does not work on Windows: also add time used by child processes
        return times[0] + times[1] + times[2] + times[3]

    def __str__(self):
        return "[%.3fs CPU, %.3fs wall-clock]" % (
            self._clock() - self.start_clock,
            time.time() - self.start_time)

    def _elapsed_cpu_time(self):
        return  self._clock() - self.start_clock

@contextlib.contextmanager
def timing(text, block=False):
    timer = Timer()
    if block:
        print("%s..." % text)
    else:
        print("%s..." % text, end=' ')
    sys.stdout.flush()
    yield
    if block:
        print("%s: %s" % (text, timer))
    else:
        print(timer)
    sys.stdout.flush()
