from typing import Any
import os
import tempfile as tmp
from contextlib import contextmanager


@contextmanager
def tempfile(suffix: str = "", dir: Any = None) -> Any:
    tf = tmp.NamedTemporaryFile(delete=False, suffix=suffix, dir=dir)
    tf.file.close()
    try:
        yield tf.name
    finally:
        try:
            os.remove(tf.name)
        except OSError as e:
            if e.errno == 2:
                pass
            else:
                raise


@contextmanager
def open_atomic(filepath: Any, *args, **kwargs) -> Any:  # type: ignore
    fsync = kwargs.pop("fsync", False)

    with tempfile(dir=os.path.dirname(os.path.abspath(filepath))) as tmppath:
        with open(tmppath, *args, **kwargs) as file:
            try:
                yield file
            finally:
                if fsync:
                    file.flush()
                    os.fsync(file.fileno())
        os.rename(tmppath, filepath)
